/**
 * A table-based representation of a Boolean function.
 * Set to be equivalent to false by default (arity = 0, output = false).
 * 
 * Creation date: (11/1/00 2:59:27 PM)
 * @author: Todd Neller
 */
public class BooleanFunction {

    /** arity of Boolean function */
    private int arity = 0;

    /** output table completely specifying Boolean function */
    private boolean[] outputTable = {false};


/**
 * BooleanFunction constructor.  Unless otherwise specified, the BooleanFunction has arity 0 and returns false.
 */
public BooleanFunction() {
    super();
}


/**
 * Create a boolean function with the specified output.
 * Given an input boolean array inputArray, f(inputArray) accesses this table according to the binary number encoded by the input array.
 * The first/last input array element corresponds to the most/significant digit in the binary number for indexing the output table.
 * 
 * Creation date: (11/1/00 3:07:35 PM)
 * @param outputTable boolean[]
 */
public BooleanFunction(boolean[] outputTable) {
    boolean badOutputTable = false;
    if (outputTable == null) {
        System.err.println("Warning: BooleanFunction constructor called with null output table.");
        badOutputTable = true;
    } else
        if (outputTable.length == 0) {
            System.err.println("Warning: BooleanFunction constructor called with length 0 output table.");
            badOutputTable = true;
        } else {
            this.outputTable = outputTable;
            arity = 0;
            int length = outputTable.length;
            while (length > 1 && (length % 2) == 0) {
                arity++;
                length = length % 2;
            }
            if (length != 1) {
                System.err.println("Warning: BooleanFunction constructor called with output table of invalid size. Size must be a power of two.");
                badOutputTable = true;
                arity = 0;
            }
        }
    if (badOutputTable)
        System.err.println("Warning: BooleanFunction constructor substituting table with single false entry.");
    else
        this.outputTable = outputTable;
}


/**
 * Create a random Boolean function of the given arity.
 *
 * Creation date: (11/1/00 3:06:03 PM)
 * @param arity int
 */
public BooleanFunction(int arity) {
    if (arity < 0) {
        System.err.println("Warning: BooleanFunction constructor called with negative arity.");
        System.err.println("Warning: BooleanFunction constructor defaulting to output table with single false entry.");
    } else {
        this.arity = arity;
        int outputTableLength = 1;
        for (int i = 0; i < arity; i++)
            outputTableLength *= 2;
        outputTable = new boolean[outputTableLength];
        java.util.Random rng = new java.util.Random();
        for (int i = 0; i < outputTableLength; i++)
            outputTable[i] = ((rng.nextInt() % 2) == 0);
    }
}


/**
 * Map input to output of Boolean function.
 * Given an input boolean array inputArray, f(inputArray) accesses the output table according to the binary number encoded by the input array.
 * The first/last input array element corresponds to the most/significant digit in the binary number for indexing the output table.
 * 
 * Creation date: (11/1/00 3:05:02 PM)
 * @return boolean
 * @param input boolean[]
 */
public boolean f(boolean[] input) {
    if (input.length != arity) {
        System.err.println("Warning: BooleanFunction.f of arity "+arity+" called with input array of length "+input.length+".");
        System.err.println("Warning: Returning false by default.");
        return false;
    }
    int entry = 0;
    int powerOfTwo = 1;
    for (int i = input.length - 1; i >= 0; i--) {
        if (input[i])
            entry += powerOfTwo;
        powerOfTwo *= 2;
    }
    return outputTable[entry];
}
/**
 * Return arity of Boolean function.
 * Creation date: (11/1/00 3:02:46 PM)
 * @return int
 */
public int getArity() {
    return arity;
}


/**
 * Output a single line with 0/1 values corresponding to true/false values of the table.
 * Creation date: (11/1/00 4:42:10 PM)
 * @return java.lang.String
 */
public String toString() {
    String s = "";
    for (int i=0; i<outputTable.length; i++)
        s += outputTable[i] ? "1" : "0";
    return s;
}
}
