/**
 * Interface for a multilayer feed-forward neural network
 * Creation date: (11/1/00 4:58:29 PM)
 * @author: Todd Neller
 */
public interface MultilayerFeedForwardNeuralNetwork {


/**
 * Use backpropogation to train network for each input/output training example.
 * Repeat this process for the given number of epochs or until converged.
 * Return the number of epochs run.
 * Boolean true/false values correspond to network values 1.0/0.0.
 *
 * Creation date: (11/1/00 5:13:42 PM)
 * @param epochs int
 * @return int number of epochs run
 */
public int backPropagationTrain(int epochs);

/**
 * Compute network outputs from given inputs (all doubles).
 *
 * Creation date: (11/1/00 5:16:34 PM)
 * @return double[]
 * @param inputs double[]
 */
public double[] computeOutputs(double[] inputs);


/**
 * Initialize multilayer feedforward network topology.
 * The first index [i] concerns which layer of links one is defining.
 * The next two indices [j][k] indicate whether output of unit j from one layer
 * has a weighted link to the input of unit k on the next layer.
 * For example, the links for a fully-connected 3-input, 8-hidden unit, 1-output network
 * would be specified by an array of length 2 which had 3x8 and 8x1 arrays with all values true.
 * Thus
 *  - the number of layers == topology.length
 *  - topology[0] is an n_0*n_1 array where n_0 is the number of inputs and n_1 is the number of units in the first layer
 *  - topology[1] is an n_1*n_2 array where n_1 and n_2 are the number of units on the first and second layer respectively
 *  - etc.
 *
 * Creation date: (11/1/00 5:04:59 PM)
 * @param topology boolean[][][]
 */
public void initializeTopology(boolean[][][] topology);


/**
 * Assign small initial random weight for all links (e.g. between -0.5 and 0.5).
 * Creation date: (11/1/00 5:06:10 PM)
 */
public void randomizeWeights();


/**
 * Set the learning rate for back-propagation algorithm.
 * Creation date: (11/1/00 5:10:58 PM)
 * @param alpha double
 */
public void setLearningRate(double alpha);


/**
 * Set the momentum constant (0.0 <= m <= 1.0) for learning.
 * This value should be 0.0 by default (no momentum).
 * Any value outside the range of 0.0-1.0 should be rejected.
 *
 * Creation date: (11/1/00 5:27:08 PM)
 * @param momentumConstant double
 */
public void setMomentumConstant(double momentumConstant);


/**
 * Set the boolean training functions for training.
 * The functions should all have the same arity as the network has inputs.
 * Each training function corresponds to a network output.
 * Training functions are boolean (true/false) but all network values are doubles (1.0/0.0).
 *
 * Creation date: (11/1/00 5:09:09 PM)
 * @param trainingFunctions BooleanFunction[]
 */
public void setTrainingFunctions(BooleanFunction[] trainingFunctions);
}
